import React, {
  cloneElement,
  ComponentProps,
  Key,
  ReactElement,
  ReactNode,
  useState,
} from 'react';
import {TableDataItem} from '../ui/tables/types/table-data-item';
import {BackendFilter} from './filters/backend-filter';
import {MessageDescriptor} from '../i18n/message-descriptor';
import {ColumnConfig} from './column-config';
import {useTrans} from '../i18n/use-trans';
import {useBackendFilterUrlParams} from './filters/backend-filter-url-params';
import {
  ResourcePaginationParams,
  usePaginatedResources,
} from './requests/paginated-resources';
import {message} from '../i18n/message';
import {DataTableContext} from './page/data-table-context';
import {AnimatePresence} from 'framer-motion';
import {ProgressBar} from '../ui/progress/progress-bar';
import {Table, TableProps} from '../ui/tables/table';
import {DataTablePaginationFooter} from './data-table-pagination-footer';
import {
  DataTableHeader,
  SelectedStateDataTableHeader,
} from './data-table-header';
import {FilterList} from './filters/filter-list/filter-list';

export interface DataTableProps<T extends TableDataItem> {
  filters?: BackendFilter[];
  columns: ColumnConfig<T>[];
  searchPlaceholder?: MessageDescriptor;
  queryParams?: Record<string, string | number | undefined | null>;
  endpoint: string;
  resourceName?: ReactNode;
  emptyStateMessage: ReactElement<{isFiltering: boolean}>;
  actions?: ReactNode;
  enableSelection?: boolean;
  selectedActions?: ReactNode;
  onRowAction?: TableProps<T>['onAction'];
  tableDomProps?: ComponentProps<'table'>;
  children?: ReactNode;
}
export function DataTable<T extends TableDataItem>({
  filters,
  columns,
  searchPlaceholder,
  queryParams,
  endpoint,
  actions,
  selectedActions,
  emptyStateMessage,
  tableDomProps,
  onRowAction,
  enableSelection = true,
  children,
}: DataTableProps<T>) {
  const {trans} = useTrans();
  const {encodedFilters} = useBackendFilterUrlParams(filters);
  const [params, setParams] = useState<ResourcePaginationParams>({perPage: 15});
  const [selectedRows, setSelectedRows] = useState<Key[]>([]);
  const query = usePaginatedResources<T>(
    endpoint,
    {
      ...params,
      ...queryParams,
      filters: encodedFilters,
    },
    {
      onSuccess: () => {
        // clear selected rows when table is reloaded from backend to avoid stale ids
        setSelectedRows([]);
      },
    }
  );

  const isFiltering = !!(params.query || params.filters || encodedFilters);
  const pagination = query.data?.pagination;
  if (!searchPlaceholder) {
    searchPlaceholder = message('Type to search...');
  }

  return (
    <DataTableContext.Provider
      value={{
        selectedRows,
        setSelectedRows,
        endpoint,
        searchPlaceholder,
        params,
        setParams,
        query,
      }}
    >
      {children}
      <AnimatePresence initial={false} mode="wait">
        {selectedRows.length ? (
          <SelectedStateDataTableHeader
            actions={selectedActions}
            key="selected"
          />
        ) : (
          <DataTableHeader actions={actions} filters={filters} key="default" />
        )}
      </AnimatePresence>
      {filters && <FilterList className="mb-14" filters={filters} />}
      <div className="relative border rounded">
        {query.isLoading && (
          <ProgressBar
            isIndeterminate
            className="absolute top-0 left-0 w-full z-10"
            aria-label={trans({message: 'Loading'})}
            size="xs"
          />
        )}

        <div className="relative overflow-x-auto md:overflow-hidden">
          <Table
            {...tableDomProps}
            data-loaded={!query.isLoading}
            columns={columns}
            data={pagination?.data || []}
            sortDescriptor={params}
            onSortChange={descriptor => {
              setParams({...params, ...descriptor});
            }}
            selectedRows={selectedRows}
            enableSelection={enableSelection}
            onSelectionChange={setSelectedRows}
            onAction={onRowAction}
          />
        </div>

        {(query.isFetched || query.isPreviousData) &&
        !pagination?.data.length ? (
          <div className="pt-50" data-testid="empty-state-message">
            {cloneElement(emptyStateMessage, {
              isFiltering,
            })}
          </div>
        ) : undefined}

        <DataTablePaginationFooter query={query} />
      </div>
    </DataTableContext.Provider>
  );
}
